@@ -268,8 +268,10 @@ TEST_F(gieTest, proj_create_crs_to_crs) {
     ASSERT_TRUE(P != nullptr);
     PJ_COORD a, b;
 
-    a.xy.x = 700000.0;
-    a.xy.y = 6000000.0;
+    a.xyzt.x = 700000.0;
+    a.xyzt.y = 6000000.0;
+    a.xyzt.z = 0;
+    a.xyzt.t = HUGE_VAL;
     b.xy.x = 307788.8761171057;
     b.xy.y = 5999669.3036037628;
 
@@ -306,8 +308,10 @@ TEST_F(gieTest, proj_create_crs_to_crs_EPSG_4326) {
     PJ_COORD a, b;
 
     // Lat, long degrees
-    a.xy.x = 0.0;
-    a.xy.y = 3.0;
+    a.xyzt.x = 0.0;
+    a.xyzt.y = 3.0;
+    a.xyzt.z = 0;
+    a.xyzt.t = HUGE_VAL;
 
     b.xy.x = 500000.0;
     b.xy.y = 0.0;
@@ -328,8 +332,10 @@ TEST_F(gieTest, proj_create_crs_to_crs_proj_longlat) {
     PJ_COORD a, b;
 
     // Long, lat degrees
-    a.xy.x = 3.0;
-    a.xy.y = 0;
+    a.xyzt.x = 3.0;
+    a.xyzt.y = 0;
+    a.xyzt.z = 0;
+    a.xyzt.t = HUGE_VAL;
 
     b.xy.x = 500000.0;
     b.xy.y = 0.0;
@@ -436,6 +442,17 @@ TEST(gie, info_functions) {
     /* we can't expect perfect numerical accuracy so testing with a tolerance */
     ASSERT_NEAR(-2.0, proj_dmstor(&buf[0], NULL), 1e-7);
 
+    /* test UTF-8 degree sign on DMS input */
+    ASSERT_NEAR(0.34512432, proj_dmstor("19°46'27\"E", NULL), 1e-7);
+
+    /* test ISO 8859-1, cp1252, et al. degree sign on DMS input */
+    ASSERT_NEAR(0.34512432,
+                proj_dmstor("19"
+                            "\260"
+                            "46'27\"E",
+                            NULL),
+                1e-7);
+
     /* test proj_derivatives_retrieve() and proj_factors_retrieve() */
     P = proj_create(PJ_DEFAULT_CTX, "+proj=merc +ellps=WGS84");
     a = proj_coord(0, 0, 0, 0);
@@ -458,6 +475,33 @@ TEST(gie, info_functions) {
 
     proj_destroy(P);
 
+    // Test with a projected CRS
+    {
+        P = proj_create(PJ_DEFAULT_CTX, "EPSG:3395");
+
+        const auto factors2 = proj_factors(P, a);
+
+        EXPECT_EQ(factors.angular_distortion, factors2.angular_distortion);
+        EXPECT_EQ(factors.meridian_parallel_angle,
+                  factors2.meridian_parallel_angle);
+        EXPECT_EQ(factors.meridian_convergence, factors2.meridian_convergence);
+        EXPECT_EQ(factors.tissot_semimajor, factors2.tissot_semimajor);
+
+        proj_destroy(P);
+    }
+
+    // Test with a geographic CRS --> error
+    {
+        P = proj_create(PJ_DEFAULT_CTX, "EPSG:4326");
+
+        const auto factors2 = proj_factors(P, a);
+        EXPECT_NE(proj_errno(P), 0);
+        proj_errno_reset(P);
+        EXPECT_EQ(factors2.meridian_parallel_angle, 0);
+
+        proj_destroy(P);
+    }
+
     /* Check that proj_list_* functions work by looping through them */
     size_t n = 0;
     for (oper_list = proj_list_operations(); oper_list->id; ++oper_list)
@@ -718,9 +762,10 @@ TEST(gie, proj_create_crs_to_crs_PULKOVO42_ETRS89) {
     proj_destroy(target_crs);
 
     // Romania
-    c.xyz.x = 45; // Lat
-    c.xyz.y = 25; // Long
-    c.xyz.z = 0;
+    c.xyzt.x = 45; // Lat
+    c.xyzt.y = 25; // Long
+    c.xyzt.z = 0;
+    c.xyzt.t = HUGE_VAL;
     c = proj_trans(P, PJ_FWD, c);
     EXPECT_NEAR(c.xy.x, 44.999701238, 1e-9);
     EXPECT_NEAR(c.xy.y, 24.998474948, 1e-9);
@@ -740,9 +785,10 @@ TEST(gie, proj_create_crs_to_crs_PULKOVO42_ETRS89) {
     EXPECT_NEAR(c.xy.x, 45, 1e-8);
     EXPECT_NEAR(c.xy.y, 25, 1e-8);
 
-    c.xyz.x = 45; // Lat
-    c.xyz.y = 25; // Long
-    c.xyz.z = 0;
+    c.xyzt.x = 45; // Lat
+    c.xyzt.y = 25; // Long
+    c.xyzt.z = 0;
+    c.xyzt.t = HUGE_VAL;
     proj_trans_generic(P, PJ_FWD, &(c.xyz.x), sizeof(double), 1, &(c.xyz.y),
                        sizeof(double), 1, &(c.xyz.z), sizeof(double), 1,
                        nullptr, 0, 0);
@@ -783,9 +829,10 @@ TEST(gie, proj_create_crs_to_crs_outside_area_of_use) {
     EXPECT_EQ(P->fwd, nullptr);
 
     // Test point outside area of use of both candidate coordinate operations
-    c.xyz.x = 58; // Lat in deg
-    c.xyz.y = 5;  // Long in deg
-    c.xyz.z = 0;
+    c.xyzt.x = 58; // Lat in deg
+    c.xyzt.y = 5;  // Long in deg
+    c.xyzt.z = 0;
+    c.xyzt.t = HUGE_VAL;
     c = proj_trans(P, PJ_FWD, c);
     EXPECT_NEAR(c.xy.x, 64.44444444444444, 1e-9); // Lat in grad
     EXPECT_NEAR(c.xy.y, 2.958634259259258, 1e-9); // Long in grad
@@ -795,6 +842,30 @@ TEST(gie, proj_create_crs_to_crs_outside_area_of_use) {
 
 // ---------------------------------------------------------------------------
 
+TEST(gie, proj_create_crs_to_crs_with_area_large) {
+
+    // Test bugfix for https://github.com/OSGeo/gdal/issues/3695
+    auto area = proj_area_create();
+    proj_area_set_bbox(area, -14.1324, 49.5614, 3.76488, 62.1463);
+    auto P =
+        proj_create_crs_to_crs(PJ_DEFAULT_CTX, "EPSG:4277", "EPSG:4326", area);
+    proj_area_destroy(area);
+    ASSERT_TRUE(P != nullptr);
+    PJ_COORD c;
+
+    c.xyzt.x = 50; // Lat in deg
+    c.xyzt.y = -2; // Long in deg
+    c.xyzt.z = 0;
+    c.xyzt.t = HUGE_VAL;
+    c = proj_trans(P, PJ_FWD, c);
+    EXPECT_NEAR(c.xy.x, 50.00065628, 1e-8);
+    EXPECT_NEAR(c.xy.y, -2.00133989, 1e-8);
+
+    proj_destroy(P);
+}
+
+// ---------------------------------------------------------------------------
+
 TEST(gie, proj_trans_generic) {
     // GDA2020 to WGS84 (G1762)
     auto P = proj_create(
